import pathlib
from typing import List

import pygame
from pygame.surface import Surface
import Colors
from Constants import FONT_FILE_PATH

class BottomBar:
    """ Bottom bar with icons and text
    """
    def __init__(self,screen: Surface,originX,originY, width, height, identifier):
        self.screen = screen
        self.x = originX
        self.y = originY
        self.width = width
        self.height = height
        self.identifier = identifier
        self.rect = (self.x,self.y,self.width,self.height)
        self.bottomBarSurface = Surface((self.width,self.height))
        
        # Fill the background
        self.bottomBarSurface.fill(Colors.BLACK)
        
        pygame.draw.rect(self.bottomBarSurface,Colors.BLACK, self.rect)
        # Initialize texts
        font = pygame.font.Font(pathlib.Path(FONT_FILE_PATH),12)
        texts = [
            'MOVE',
            'START',
            'HOME',
            'SWITCH VIEW',
            'CREDITS'
        ]

        fontsList: List[Surface] = []
        for text in texts:
            fontItem = font.render(text,True,Colors.WHITE)
            fontsList.append(fontItem)

        # Initialize icons
        iconPaths = [
            './assets/bottom_bar/joystick_a.PNG',
            './assets/bottom_bar/button_single.PNG',
            './assets/bottom_bar/button_home.PNG',
            './assets/bottom_bar/button_yellow.PNG',
            './assets/bottom_bar/button_red.PNG',
        ]

        iconsList: List[Surface] = []
        iconsRectList: List[pygame.rect.Rect] = []
        for i in range(len(iconPaths)):
            icon = pygame.image.load(iconPaths[i])
            icon = pygame.transform.scale(icon,(35,35))
            iconsList.append(icon)
            iconRect = icon.get_rect()
            iconRect.centery = self.height/2
            iconsRectList.append(iconRect)
        
        # Draw icons and texts on surface
        leftStart = self.x + 6
        iconsRectList[0].left = leftStart
        self.bottomBarSurface.blit(iconsList[0],iconsRectList[0])
        self.bottomBarSurface.blit(fontsList[0],(iconsRectList[0].right,self.height/2 - fontsList[0].get_height()/2))

        for i in range(1,5):
            leftStart = iconsRectList[i-1].right + fontsList[i-1].get_rect().right + 38
            iconsRectList[i].left = leftStart
            self.bottomBarSurface.blit(iconsList[i],iconsRectList[i])
            self.bottomBarSurface.blit(fontsList[i],(iconsRectList[i].right ,self.height/2 - fontsList[i].get_height()/2))


    
    def render(self):
        
        self.screen.blit(self.bottomBarSurface,self.rect)
        
