from typing import List

from pygame.surface import Surface
from pygame.rect import Rect
from CarouselIndicatorItem import CarouselIndicatorItem
import Colors

class CarouselIndicators:
    def __init__(self,screen: Surface, identifier:str,currentIndex:int,size:int) -> None:
        self.screen = screen
        
        self.identifier = identifier
        self.currentIndex = currentIndex
        self.size = size
        self.width = 33*self.size + 10*(self.size -1)
        self.height = 50

        self._surface = Surface((self.width,self.height))
        self._rect = self._surface.get_rect()

        # Fill the background
        self._surface.fill(Colors.BLACK)
        
        # Position the Carousel Indicators
        self._rect.top = 0
        self._rect.centerx = self.screen.get_rect().centerx

        self.items: List[CarouselIndicatorItem] = []


        for i in range(size):
            indicator = CarouselIndicatorItem(self._surface)
            indicator.rect.left = i*(33+10)
            indicator.rect.centery = self._rect.centery
            self._surface.blit(indicator.surface,indicator.rect)
            self.items.append(indicator)

        self.screen.blit(self._surface,self._rect)

    @property
    def surface(self) -> Surface:
        return self._surface
    
    @property
    def rect(self) -> Rect:
        return self._rect

    def update(self,currentIndex:int) -> None:
        self.currentIndex = currentIndex
        for index,indicator in enumerate(self.items):
            self._surface.blit(indicator.surface,indicator.rect,indicator.rect)
            
            # print('update',index,self.currentIndex,indicator.rect)
            
            if index == self.currentIndex:
                indicator.activate()
            else:
                indicator.deactivate()
            
            indicator.rect.left = index*(33+10)
            indicator.rect.centery = self._rect.centery

            self._surface.blit(indicator.surface,indicator.rect)

        # self.screen.blit(self._surface,self._rect)

    def render(self) -> None:
        self.screen.blit(self._surface,self._rect,self._rect)
        self.screen.blit(self._surface,self._rect)