from typing import List
import pathlib
import pygame

from pygame.surface import Surface

from Colors import WHITE,BLACK
from Constants import FONT_FILE_PATH
from Manifest import Manifest

class CarouselItem:
    """ A Carousel Slide Item
    """
    def __init__(self,screen:Surface,manifest:Manifest):
        self.screen = screen
        self.image = manifest.cover_art
        self.gameName = manifest.name
        self.title = manifest.name+' ('+manifest.year+')'

        # Set sizes of rectangles
        sl = screen.get_width()/12
        frec = 4.9*sl
        brec = 4.2*sl

        # Surface if focused
        self._focusSurface = Surface((frec,frec))
        self._focusRect = pygame.draw.rect(self._focusSurface,WHITE,(0,0,frec,frec),width=5)
        self._focusSurface.blit(pygame.transform.smoothscale(pygame.image.load(self.image).convert(),(frec-10,frec-10)),(5,5,frec-5,frec-5))
        # Surface if blurred 
        self._blurSurface = pygame.transform.smoothscale(pygame.image.load(self.image).convert(),(brec,brec))
        self._blurRect = self._blurSurface.get_rect()


        self._surface = self._blurSurface.copy()
        self._rect = self._blurRect
        
        
        self._backupSurface = self._surface.copy()
        

        # Caption and Text Surface
        self._captionSurface = Surface((frec-10,24))
        self._captionSurface.set_alpha(180)
        self._captionRect = pygame.draw.rect(self._captionSurface,BLACK,(0,0,frec-10,24))

        font = pygame.font.Font(pathlib.Path(FONT_FILE_PATH),14)
        text = font.render(self.title,True,WHITE)
        self._captionSurface.blit(text,(self._captionRect.centerx - text.get_width()/2,self._captionRect.height/2 - text.get_height()/2))
        self._captionRect.bottom = frec-5
        self._captionRect.left = 5
        
        # Draw the Caption on the focus surface
        self._focusSurface.blit(self._captionSurface,self._captionRect)
        

    @property
    def surface(self):
        return self._surface
    
    @property
    def rect(self):
        return self._rect
    
    @surface.setter
    def surface(self,s):
        self._surface = s

    @rect.setter
    def rect(self,r):
        self._rect = r
    
        
    def focus(self):
        self._focusRect.center = self._rect.center
        self._surface = self._focusSurface
        self._rect = self._focusRect
        
        
        
    def blur(self):
        self._blurRect.center = self._rect.center
        self._surface = self._blurSurface
        self._rect = self._blurRect
        

    def onSelect(self,callback=None,cargs=()):
        data = [1,2,3,4]
        print('Carousel Item onSelect')
        if callback is not None:
            callback(cargs,self.gameName)