import pygame
import Colors
from Constants import FONT_FILE_PATH
from pygame.surface import Surface
from pygame.locals import *


class Description:
    """ The Description
        Consists of title, description of a game, group members 
    """
    def __init__(self, surface: Surface, x: int, y: int, width: int, height: int, gridRect: Rect):

        self.surface = surface
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.grid_rect = gridRect


        # Fonts
        self.font = pygame.font.Font(FONT_FILE_PATH, 10)
        self.fontTitle = pygame.font.Font(FONT_FILE_PATH, 12)
        self.fontGroup = pygame.font.Font(FONT_FILE_PATH, 7)
        
        self.description_border_x = 10
        self.description_border_y = 5
        # The underlying surface
        self.infoSurface = Surface((self.width,self.height))
        self.infoSurfaceRect = pygame.rect.Rect(self.x, self.grid_rect.y, self.infoSurface.get_width(), self.grid_rect.bottom - self.grid_rect.y+10)
        # The title surface
        self.titleRect = pygame.rect.Rect(0,self.description_border_y, self.infoSurface.get_rect().width - 2*self.description_border_x,20)
        self.titleRect.centerx = self.infoSurface.get_rect().centerx
        self.title = pygame.draw.rect(self.infoSurface, Colors.DARK_GRAY, self.titleRect)
        # The description surface
        self.infoRect = pygame.rect.Rect(0, self.title.bottom + self.description_border_y, self.infoSurface.get_rect().width - 2*self.description_border_x,self.infoSurface.get_rect().height - self.title.height - 2*self.description_border_y - 4*6)
        self.infoRect.centerx = self.infoSurface.get_rect().centerx
        self.info = pygame.draw.rect(self.infoSurface, Colors.DARK_GRAY, self.infoRect)

    def render(self, name, description, groupMembers):
 
        self.infoSurface.fill(Colors.ANOTHER_GRAY)
        # Draw the text on the surfaces
        drawText(self.infoSurface, name, Colors.WHITE, self.title, self.fontTitle)
        drawText(self.infoSurface, description, Colors.WHITE, self.info, self.font)

        # Draw names on surface
        for ix, member in enumerate(groupMembers):
            group = self.fontGroup.render(member, True, Colors.LIGHT_GRAY)
            self.infoSurface.blit(group, (self.infoSurfaceRect.width-group.get_width()-self.description_border_x, self.infoSurfaceRect.bottom -55- (len(groupMembers)-ix)* 8))

        
    
        self.surface.blit(self.infoSurface, self.infoSurfaceRect)









#https://www.pygame.org/wiki/TextWrap
# draw some text into an area of a surface
# automatically wraps words
# returns any text that didn't get blitted
def drawText(surface, text, color, rect, font, aa=False, bkg=None):
    rect = Rect(rect)
    y = rect.top
    lineSpacing = 5

    # get the height of the font
    fontHeight = font.size("Tg")[1]

    while text:
        i = 1

        # determine if the row of text will be outside our area
        if y + fontHeight > rect.bottom:
            break

        # determine maximum width of line
        while font.size(text[:i])[0] < rect.width and i < len(text):
            i += 1

        # if we've wrapped the text, then adjust the wrap to the last word      
        if i < len(text): 
            i = text.rfind(" ", 0, i) + 1

        # render the line and blit it to the surface
        if bkg:
            image = font.render(text[:i], 1, color, bkg)
            image.set_colorkey(bkg)
        else:
            image = font.render(text[:i], aa, color)

        surface.blit(image, ((rect.width/2 - image.get_width()/2)+15, y))
        y += fontHeight + lineSpacing

        # remove the text we just blitted
        text = text[i:]

    return text