import pygame
from pygame.surface import Surface
from pygame.rect import Rect
from pygame.font import Font
from Constants import SCREEN_HEIGHT,SCREEN_WIDTH,FONT_FILE_PATH
import Colors
from Controls import Controls

class ErrorImg(pygame.sprite.Sprite):
    def __init__(self, pos_x, pos_y):
        super().__init__()
        
        img1 = pygame.image.load("./assets/error/e1.png")
        img2 = pygame.image.load("./assets/error/e2.png")
        
        # load images        
        self.sprites = []
        self.sprites.append(img1)
        self.sprites.append(img2)
        self.current_sprite = 0
        self.image = self.sprites[self.current_sprite]

        # rectangle and position it
        self.rect = self.image.get_rect()
        self.rect.topleft = [pos_x,pos_y]

        self.framecount = 0   
        
    # animate the sprites
    def update(self):
        self.framecount += 1
        
        if self.framecount % 30 != 0:
           return 
        
        self.current_sprite += 1
        
        if self.current_sprite >= len(self.sprites):
            self.current_sprite = 0
        
        self.image = self.sprites[self.current_sprite]

class ErrorScreen():
    def __init__(self,screen) -> None:
        self.screen = screen
        self.surface = Surface((SCREEN_WIDTH,SCREEN_HEIGHT))
        self.rect = self.surface.get_rect()
        self.title = "ERROR"
        self.message = "PLEASE CONTACT SUPPORT"
        self.title_font = Font(FONT_FILE_PATH,20)
        self.subtitle_font = Font(FONT_FILE_PATH,14)
        self.title_font_surface = self.title_font.render(self.title,True,Colors.RED)
        self.subtitle_font_surface = self.subtitle_font.render(self.message,True,Colors.RED)
        

    def render(self,title='',message=''):
        if (title!= '' or message != ''):
            self.title = title
            self.message = message
            self.title_font_surface = self.title_font.render(self.title,True,Colors.RED)
            self.subtitle_font_surface = self.subtitle_font.render(self.message,True,Colors.RED)
        
        self.title_rect = self.title_font_surface.get_rect()
        self.subtitle_rect = self.subtitle_font_surface.get_rect()
        self.title_rect.centerx = self.rect.centerx
        self.subtitle_rect.centerx = self.rect.centerx
        self.title_rect.top = 40
        self.subtitle_rect.top = self.title_rect.bottom + 10
        
        self.surface.blit(self.title_font_surface,self.title_rect)
        
        self.surface.blit(self.subtitle_font_surface,self.subtitle_rect)
        self.screen.blit(self.surface,(0,0))

# Errorscreen sprites
errorscreen_sprites = pygame.sprite.Group()
errorscreen = ErrorImg(0, 0)
errorscreen_sprites.add(errorscreen)

def handleError(screen:Surface,controls: Controls,callback=None):
    """Creates a static error screen with a generic warning

    Args:
        screen (Surface): the screen
        controls (Controls): Controls
        callback (_type_, optional): Callback to be called. Defaults to None.
    """
    for i in range(100):
        # ErrorScreen
        errorscreen_sprites.draw(screen)
        errorscreen_sprites.update()
        pygame.display.update() # Refresh the onscreen display
    while True:
        for event in pygame.event.get():
            if controls.is_home_button(event):
                if callback is not None:
                    callback()
                    return

def handleExceptionScreen(screen:Surface,controls:Controls,title='',message=''):
    """Creates a dynamic error screen whose message can be set.

    Args:
        screen (Surface): default screen
        controls (Controls): Controls
        title (str, optional): Title of the error screen. Defaults to ''.
        message (str, optional): Message of the error screen. Defaults to ''.
    """
    dynamic_error = ErrorScreen(screen)
    for _ in range(100):
        dynamic_error.render(title,message)
        pygame.display.flip()
    while True:
        for event in pygame.event.get():
            if controls.is_button_down(event):
                return
            
