from typing import List
from json import load
from os.path import join, exists,isfile
import os


class Manifest:
    start: str
    name: str
    version: str
    icon: str
    cover_art: str
    image: str
    is_multiplayer: bool
    description: str
    install: str
    preinstall: str
    group_members: List[str]

    def __init__(
            self,
            start: str = '',
            name: str = 'Game',
            version: str = '0.0.0',
            year: str = '2023',
            cover_art: str = '',
            image: str = '',
            is_multiplayer: bool = False,
            description: str = '',
            install: str = '',
            preinstall: str = '',
            group_members: List[str] = ['a', 'b'],
            is_legacy=None,
            parent_directory: str = '.'
    ) -> None:
        #if (cover_art.startswith('./')):
        #    self.parent_directory = '.'
        #else:
        self.parent_directory = parent_directory
        self.start = start
        self.name = name
        self.version = version
        if year is None:
            year = '2023'
        self.year = year
        self.cover_art = './assets/bottom_bar/joystick.PNG'
        if isfile(join(parent_directory,cover_art)):
            self.cover_art = join(parent_directory,cover_art)
        elif cover_art.startswith('./'):
            self.cover_art = cover_art
        
        self.image = './assets/bottom_bar/joystick.PNG'
        if isfile(join(parent_directory,image)):
            self.image = join(parent_directory,image)
        elif image.startswith('./'):
            self.image = image
            
        self.is_multiplayer = is_multiplayer
        self.description = description
        self.install = install
        self.preinstall = preinstall
        self.group_members = group_members
        if is_legacy is None:
            is_legacy = False
        self.is_legacy = is_legacy
        

def from_file(manifestFile,parentDir):
    manifestJson = load(manifestFile)
    print(parentDir)
    return Manifest(
        manifestJson.get('start'),
        manifestJson.get('name'),
        manifestJson.get('version'),
        manifestJson.get('year'),
        manifestJson.get('coverArt'),
        manifestJson.get('image'),
        manifestJson.get('isMultiplayer'),
        manifestJson.get('description'),
        manifestJson.get('install'),
        manifestJson.get('preinstall'),
        manifestJson.get('groupMembers'),
        manifestJson.get('isLegacy'),
        parentDir
    )


def import_manifests(gamesPath) -> List[Manifest]:
    manifests: List[Manifest] = []
    for subdir, dirs, files in os.walk(gamesPath):
        for dir in dirs:
            path = join(gamesPath, dir, 'manifest.json')
            print(path)
            try:
                if not exists(path):
                    raise FileNotFoundError(path)
                with open(path, 'r', encoding='utf-8') as file:
                    manifests.append(from_file(file,join(gamesPath,dir)))
            except FileNotFoundError:
                print(f'File not found {path}')
        break

    if not len(manifests) > 0:
        raise Exception('No games are loaded. Please load games in games/')
    return manifests
