import os
import signal
import subprocess
from queue import Queue
from Manifest import Manifest
import shlex
import threading
import pygame
from pygame.surface import Surface
from Loop import LoopState
from State import MenuStates
from Manifest import Manifest
from typing import List,Tuple,Callable
from Controls import Controls
from Loadingscreen import *
from Colors import BLACK
from Errorscreen import handleError
from Constants import FRAME_RATE

# Loadingscreen sprites
loading_sprites = pygame.sprite.Group()
joy = JoystickImg(0, 0)
loading_sprites.add(joy)

process_queue:Queue[subprocess.Popen] = Queue(1)


def handle_terminate_process():
    """
    Handle the termination of a process/process group
    """
    if not process_queue.empty():
        # Get the the game from the process queue and kill it
        runningGame = process_queue.get()
        print('running Game killed',runningGame)
        # Doing this to also kill processes launched from a shell script
        os.killpg(os.getpgid(runningGame.pid),signal.SIGTERM)
        #runningGame.terminate()

def start_process_thread(game_manifest:Manifest,Loop:LoopState,State:MenuStates):
    """
    Starts a process thread. Just starting a process on a seperate thread.
    
    Args:
        game_manifest (Manifest): Manifest of the game to be launched
        Loop (LoopState): Loop state class
        State (MenuStates): Menu state class
    """
    # Start the specific vitual environment and launch the game using a single command
    cmd = game_manifest.start.lstrip()
    if game_manifest.start.lstrip().startswith('python'):
        cmd = '.venv/bin/' + game_manifest.start.lstrip()
    args = shlex.split(cmd)
    print('game dir',game_manifest.parent_directory)
    with subprocess.Popen(args, stdout=subprocess.PIPE,cwd=game_manifest.parent_directory,shell=False,preexec_fn=os.setsid) as proc:
        process_queue.put(proc)
        print('process_thread',process_queue)
        print(proc.stdout.read())
        retcode = proc.wait()
        if retcode:
            if not process_queue.empty():
                # Empty the process queue
                game = process_queue.get()
                print('process exits on its own.',game,proc)
                # Terminate the process regardless of whether the process exits or not.
                game.terminate()
                # os.killpg(os.getpgid(game.pid),signal.SIGTERM)
            Loop.setUI()
            State.go_to_home()


def start_the_game(cargs:Tuple[MenuStates,LoopState,Surface,List[Manifest],Controls,Callable]=(),data=''):
    """
    Function to start a game. starts a new game when called with the appropriate arguments and data.
    The game can either be started in legacy mode or in normal execution mode.
    The cargs should have the menu state, loop state, the display screen, the manifest list containing the games,controls class,
    A callable to called if an error happens.

    Args:
        cargs (Tuple[MenuStates,LoopState,Surface,List[Manifest],Controls,Callable], optional): Pass the requisite variables: 
        the menu state, loop state, the display screen, the manifest list containing the games,controls class, A callable to called if an error happens.. Defaults to ().
        
        data (str, optional): pass the game name/data here. Defaults to ''.
    """
    # Do the job here !
    State, Loop,screen,manifestList,controls,callback = cargs
    # print('args',*cargs)
    # Set the next states for the menu
    State.go_to_game_state()
    Loop.setGame()
    # Erase the screen
    screen.fill(BLACK)

    print('start the game ', data)
    selected_game_manifest = None
    # Search for the game in the manifest list
    for manifestObj in manifestList:
        if data == manifestObj.name:
            selected_game_manifest = manifestObj
    
    # start the selected game
    if selected_game_manifest is not None:
        if selected_game_manifest.is_legacy:
            print('Enabling Legacy mode...')
            Loop.setLegacy()
    
        # Loading Screen
        for _ in range(FRAME_RATE):
            loading_sprites.draw(screen)
            loading_sprites.update()
            pygame.display.flip() # Refresh the onscreen display

        try:
            # Start the process from a new thread so that the main UI thread is not blocked
            process_thread = threading.Thread(target=start_process_thread,args=(selected_game_manifest,Loop,State))
            process_thread.start()
            # process_thread.join()
        except Exception as error:
            print(error)
            handleError(screen,controls,callback)
    else:
        # Handle an error
        handleError(screen,controls,callback)
        
