import pygame
import Colors
from pygame.surface import Surface
from pygame.locals import *
from pygame import draw

class ScrollBar:
    """ Scroll bar with slider
    """
    def __init__(self, surface: Surface, y:float, default_width:float, list_length:int, animation_frames:int):
        
        self.surface = surface
        self.y = y
        self.default_width = default_width
        self.list_length = list_length
        self.total_animation_frames = animation_frames

        # Initialize scrollbar so that the animation works with varying number of animation frames
        self.scrollBarWidth = round(default_width/self.list_length/self.total_animation_frames) * self.list_length * self.total_animation_frames
        self.scrollBarSurface = Surface((self.scrollBarWidth, 5))
        self.scrollBar = draw.rect(self.scrollBarSurface, Colors.DARK_GRAY, (0,self.y,self.scrollBarWidth,5), 0,3)
        self.scrollBar.centerx = int((self.surface.get_rect().width/2)-(self.scrollBarWidth/2))
        self.scrollBarSurface.fill(Colors.DARK_GRAY)
       
        #Initialize the slider of the scrollbar
        self.sliderSurface = Surface(((self.scrollBarSurface.get_width()/self.list_length),self.scrollBarSurface.get_height()))
        self.sliderRect = draw.rect(self.sliderSurface, Colors.WHITE, (245,40, (self.scrollBarSurface.get_width()/self.list_length), self.scrollBarSurface.get_height()),0,3)
        self.sliderRect.topleft = self.scrollBar.topleft
        self.slider = draw.rect(self.sliderSurface, Colors.WHITE, (0,0, (self.scrollBarSurface.get_width()/self.list_length),self.scrollBarSurface.get_height()),0,3)
    
    
    def render(self):
        #Draw the scrollbar and slider on the surface
        self.surface.blit(self.scrollBarSurface, self.scrollBar)
        self.surface.blit(self.sliderSurface, self.sliderRect)


    def animateSlider(self, newPosx, activeIndex):
        #Move the slider according to the direction in which the carousel items are moving
        if newPosx >0:
            if activeIndex == self.list_length-1:
                sliderSpeed = [int(((self.scrollBarSurface.get_width()/(self.list_length+1))/self.total_animation_frames) * (self.list_length)),0]
            else:
                sliderSpeed = [-int((self.scrollBarSurface.get_width()/(self.list_length))/self.total_animation_frames),0]
        else:
            if activeIndex == 0:
                sliderSpeed = [-int(((self.scrollBarSurface.get_width()/(self.list_length+1))/self.total_animation_frames) * (self.list_length)),0]
            else:
                sliderSpeed = [int((self.scrollBarSurface.get_width()/(self.list_length))/self.total_animation_frames),0]
        

        self.surface.blit(self.sliderSurface, self.sliderRect)
        self.sliderRect = self.sliderRect.move(sliderSpeed)
        self.surface.blit(self.sliderSurface, self.sliderRect) 