"""
Fabarcade Menu
Created by MCP Group 2 
Authors: Shailesh Iyer, Sijia Guo,Lina Gönnheimer,Katharina Güths
=========== DISCLAIMER ==============================================================
Provided as is. May contain bugs with your game so make sure to test on the fabarcade.
=====================================================================================
"""
import signal
import pygame
import sys
import os
import subprocess

import json
import threading
from typing import List as TList
from traceback import print_exception
import asyncio
from queue import Queue
from pygame import mixer
import threading

from pygame.locals import *
import cProfile, pstats
from Loop import LoopState
from State import MenuStates
from Constants import *
from Colors import *

pygame.init()
size = width,height = SCREEN_WIDTH, SCREEN_HEIGHT


mainClock = pygame.time.Clock()
screen = pygame.display.set_mode(size, flags=pygame.SCALED, vsync=1)
pygame.display.set_caption('Fabarcade Menu')


from Grid import Grid
from Carousel import Carousel
from BottomBar import BottomBar
from List import List
from Errorscreen import *
from Splashscreen import *
from CreditsScreen import *
from Bootup import *
from FPSCounter import FPSCounter
from Controls import Controls,AXIS_THRESHOLD
from Manifest import Manifest, import_manifests
from Debounce import Debounce
from LegacyControls import LegacyControls
from ProcessHandler import handle_terminate_process,start_the_game

# Profiler
profiler = cProfile.Profile()

from screenTransitions import *
from HomeButton import HomeButton

# Loading the font
font = pygame.font.Font(FONT_FILE_PATH, 40)
# Keeping track of which loop we are supposed to be in.
# Whether we are in UI Loop, Background Loop or Legacy Loop
# Or are we there in credits loop. These 4 Loops form the backbone
# Of our menu.
Loop = LoopState()
controls = Controls()

# List of Game Manifests shown in the menu.
manifestList:TList[Manifest] = []


# Keeping track of the current menu state. 
# whether It is carousel,grid or a game is running.
State = MenuStates()

# load bgm and sfx
mixer.music.load("./assets/music/Bobs-Adventures-back34.ogg")
loadingsfx = mixer.Sound("./assets/music/sfx/LoadSave.wav")
# mixer.music.play(-1)

# Creating the sprites and groups
# Bootscreen sprites
boot_sprites = pygame.sprite.Group()
boot = BootImg(0, 0)
boot_sprites.add(boot)

# Splashscreen sprites
splash_sprites = pygame.sprite.Group()
splash = SplashImg(0, 0)
splash_sprites.add(splash)

# Handling exit on signal
def exit_handler(sig, frame):

    pygame.quit()
    sys.exit()


# Change handlers to execute on Item Change in carousel and grid
def onChangedHandler(carousel: Carousel):
    print('on Changed Handler current index is', carousel.activeIndex)
    State.set_index(carousel.activeIndex)
    print(State.get_index())

def gridChangedHandler(grid: Grid):
    print('Grid Changed Handler current index is', grid.activeIndex)
    State.set_index(grid.activeIndex)
    print(State.get_index())

# changes the view from carousel to grid or vice versa while preserving the selected game.
# is called upon pressing the yellow button
def switch_view(carousel,grid):
    if(State.is_carousel()):
        grid.setActiveIndex(State.get_index())
    elif(State.is_grid()):
        carousel.setActiveIndex(State.get_index())
    State.next_state()
    print(State.get_index())
    
#  default loop that is running when the menu is shown. Reacts to input from buttons and joysticks
def UILoop():
    # play the menu music
    mixer.music.play(-1)

    # initialize the views of the menu as well as the bottombar, which is shown in both views
    carousel = Carousel(screen, 0, 0, width, height - 70,
                        'carousel', manifestList, onChanged=onChangedHandler)
    grid = Grid(screen, width, height -70, manifestList, 'grid',onChanged=gridChangedHandler)
    bottomBar = BottomBar(screen, 0, height-70, width, 70, 'bottomBar')
    # initialize the clock and debouncer for correct input recognition
    fpsCounter = FPSCounter(screen, clock=mainClock)
    deb = Debounce()
    # Clear the event loop before (re)entering the loop for input recognition
    pygame.event.clear()
    # setting the previously selected game to be shown as selected
    carousel.setActiveIndex(State.get_index())
    INPUT_DEBOUNCE = 10
    while Loop.isUI():
        # Listen to Window Events/ Process Inputs
        
        for event in pygame.event.get():
            # handle input concerning exit, additional joysticks, credits and home button
            if event.type == pygame.QUIT:
                Loop.quit()
            elif event.type == pygame.JOYDEVICEADDED:
                controls.add_joystick(event)
            elif event.type == pygame.JOYDEVICEREMOVED:
                controls.remove_joyStick(event)
            elif controls.is_red_button(event):
                Loop.setCredits()
            elif controls.is_home_button(event):
                print('home pressed')
                Loop.quit()
        
        # move the cursor in the current view according to received input while using a debouncer
        controls.get_state()
        if (controls.get_horizontal_axis() > AXIS_THRESHOLD):
            if (State.is_carousel()):
                carousel.next()
            elif(State.is_grid()):
                deb.debounce(grid.cursorRight,INPUT_DEBOUNCE)
        elif (controls.get_horizontal_axis() < -AXIS_THRESHOLD):
            if (State.is_carousel()):
                carousel.prev()
            elif(State.is_grid()):
                deb.debounce(grid.cursorLeft,INPUT_DEBOUNCE)
        
        if (controls.get_vertical_axis() > AXIS_THRESHOLD):
            if(State.is_grid()):
                deb.debounce(grid.cursorDown,INPUT_DEBOUNCE)
        elif (controls.get_vertical_axis() < -AXIS_THRESHOLD):
            if(State.is_grid()):
                deb.debounce(grid.cursorUp, INPUT_DEBOUNCE)                
        if (controls.get_yellow_button()):
            deb.debounce(switch_view,INPUT_DEBOUNCE,(carousel,grid))

        # start the selected game when the start button is pressed
        if (controls.get_start_button()):
            # stop music when game is started
            mixer.music.stop()
            # loading sfx and starting the game
            loadingsfx.play()
            if (State.is_carousel()):
                deb.debounce(carousel.onSelect,INPUT_DEBOUNCE,(start_the_game, (State, Loop,screen,manifestList,controls,OnHomeButtonPressed)))
            elif(State.is_grid()):
                deb.debounce(grid.onSelect,INPUT_DEBOUNCE,(start_the_game, (State, Loop,screen,manifestList,controls,OnHomeButtonPressed)))

            return
                
        # Do Logical Updates here
        deb.wait()
        # Fill the Screen with Solid Color
        screen.fill(BLACK)

        # Render the UI 
        # Render the bottom bar
        bottomBar.render()

        if(State.is_carousel()):
            # Render Carousel
            carousel.render()
        elif(State.is_grid()):
            # Render Grid
            grid.render()

        # refresh the onscreen display
        pygame.display.flip()  
        # set the framerate
        mainClock.tick(FRAME_RATE)    

# returns to the carousel view, is called upon pressing the home button
def OnHomeButtonPressed():
    State.go_to_home()
    Loop.setUI()
    pygame.event.clear()

# show the boot screen and load the manifest of all games
def boot():
    global manifestList
    try:
        boot_sprites.draw(screen)
        boot_sprites.update()  
        pygame.display.update() # Refresh the onscreen display

        manifestList = import_manifests(GAMES_PATH)
        for i in range(100):
            # Bootscreen
            boot_sprites.draw(screen)
            boot_sprites.update()  
            pygame.display.update() # Refresh the onscreen display

    # print an error message if no games are present in the games folder
    except Exception as e:
        print('no games found...')
        raise e.with_traceback(e.__traceback__)
    
# show the splash screen until any button is pressed. Is called after start-up
def splash():
    buttonPressed = False
    pygame.event.clear(KEYDOWN)
    pygame.event.clear(JOYBUTTONDOWN)
    while(not buttonPressed):
    # Splashscreen
        splash_sprites.draw(screen)
        splash_sprites.update()
        pygame.display.update() # Refresh the onscreen display
        for e in pygame.event.get(eventtype=[KEYDOWN,JOYBUTTONDOWN]):
            if(controls.is_button_down(e)):
                buttonPressed = True

# listen to input on the home button only. Is called repeatedly while any of the new games is run.
# return to the menu once the home button is pressed
def backgroundLoop():
    print("background")
    for event in pygame.event.get():
        print(event)
        if controls.is_home_button(event):
            handle_terminate_process()
            OnHomeButtonPressed()
            
    # Run the background loop only once every second.
    mainClock.tick(1) 

# start an asynchronous loop for the selected legacy game to run in and emulate all button 
# input for that game.
def legacyLoop(legacy_controls:LegacyControls):
    print("legacy")

    pygame.event.set_allowed(HOME_BUTTON_DOWN)
    pygame.event.set_allowed(HOME_BUTTON_UP)
    pygame.event.set_allowed(JOYAXISMOTION)
    pygame.event.set_allowed(JOYBUTTONDOWN)
    pygame.event.set_allowed(JOYBUTTONUP)
    
    event_loop = asyncio.new_event_loop()
    try:
        asyncio.set_event_loop(event_loop)
        asyncio.ensure_future(legacy_controls.emulate(Loop,OnHomeButtonPressed,event_loop))
        event_loop.run_forever()
        for t in [t for t in legacy_controls.tasks_list if not (t.done() or t.cancelled())]:
            # give canceled tasks the last chance to run
            event_loop.run_until_complete(t)
    finally:
        event_loop.close()
    
    print('exits emulation...')

# show the credits 
def creditsLoop(): 
    credits = Credits(screen, font, mainClock, 1)
    while(Loop.isCredits()):
        screen.fill(BLACK)
        credits.render()
        # Refresh the onscreen display
        pygame.display.flip() 
        
        # listen to input on the joysticks to accelerate or slow down the movement of the credits
        for e in pygame.event.get(eventtype=[KEYDOWN,JOYBUTTONDOWN]):
            if(controls.is_button_down(e)):
                if(controls.is_movement_down(e)):
                    credits.accelerate()
                elif(controls.is_movement_up(e)):
                    credits.slowDown()
                else:
                    # return to the menu when any other button is pressed
                    Loop.setUI()
        
# main event loop. Is called upon start-up
def main():
    print('main')
    signal.signal(signal.SIGINT, exit_handler)
    # Initialise the home button 
    home = HomeButton()
    # Initialise legacy controls
    legacy_controls = LegacyControls(controls)
    # verify the functionality of the home button
    if TEST_HOME_BUTTON:
        h = threading.Thread(target=home.test, args=(), daemon=True)
        h.start()
    
    pygame.event.clear()
    # activate fullscreen if desired
    if IS_FULLSCREEN:
        pygame.display.toggle_fullscreen()
    try:
        # render boot and splash screen in the beginning
        boot()
        splash()
        # start the main loop and choose the right action depending on the current state
        Loop.setUI()
        while not Loop.isQuitting():
            print("main" + Loop.loopvar)
            if(Loop.isUI()):
                UILoop()
            elif(Loop.isGame()):
                backgroundLoop()
            elif(Loop.isLegacy()):
                legacyLoop(legacy_controls)
            elif(Loop.isCredits()):
                creditsLoop()
            elif(Loop.isQuitting()):
                return
            else:
                handleError(screen,controls,OnHomeButtonPressed)
    # print error message if any exception occurs
    except Exception as e:
        print(str(e.with_traceback(e.__traceback__)))
        print_exception(e)
        handleExceptionScreen(screen,controls,'ERROR',str(e))  

# entry point of the program
if __name__ == '__main__':
    # initialize performance debugging
    perfomanceDebug = False
    for arg in sys.argv:
        if arg == '--perf-debug':
            perfomanceDebug = True
    
    if perfomanceDebug:
        profiler.enable()
    
    # start the main event loop
    main()
    
    # tear down
    if perfomanceDebug:
        profiler.disable()
        stats = pstats.Stats(profiler).sort_stats('cumtime')
        stats.dump_stats('./perf-data.prof')
    
    pygame.quit()
    raise SystemExit