import unittest
import pygame
from .context import Controls


class TestControlClass(unittest.TestCase):
    @classmethod
    def setUpClass(self) -> None:
        self.controls = Controls.Controls()

        # Add Mock Joysticks

        # e = pygame.event.Event(pygame.JOYDEVICEADDED,{'device_index':1001})
        # self.controls.addJoystick(e)
        # e = pygame.event.Event(pygame.JOYDEVICEADDED,{'device_index':1002})
        # self.controls.addJoystick(e)

        self.controls.joystickInstanceIdArray.append(1001)
        self.controls.joystickInstanceIdArray.append(1002)

    @classmethod
    def tearDownClass(self) -> None:

        # Remove Mock Joysticks
        # e = pygame.event.Event(pygame.JOYDEVICEREMOVED,{'instance_id':1001})
        # self.controls.addJoystick(e)
        # e = pygame.event.Event(pygame.JOYDEVICEREMOVED,{'instance_id':1002})
        # self.controls.addJoystick(e)
        self.controls.joystickInstanceIdArray.clear()

        # De-Initalise class
        self.controls = None

    def test_is_button_down(self):
        e1 = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_w})
        self.assertTrue(self.controls.is_button_down(e1))
        e2 = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 9})
        self.assertTrue(self.controls.is_button_down(e2))
        e3 = pygame.event.Event(pygame.QUIT, {'button': 9})
        self.assertFalse(self.controls.is_button_down(e3))

    def test_is_button_up(self):
        e1 = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_w})
        self.assertTrue(self.controls.is_button_up(e1))
        e2 = pygame.event.Event(pygame.JOYBUTTONUP, {'button': 9})
        self.assertTrue(self.controls.is_button_up(e2))
        e3 = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_w})
        self.assertFalse(self.controls.is_button_up(e3))

    def test_is_movement(self):
        e1 = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_w})
        self.assertTrue(self.controls.is_movement(e1))
        e2 = pygame.event.Event(pygame.JOYAXISMOTION, {'button': 9})
        self.assertTrue(self.controls.is_movement(e2))
        e3 = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_w})
        self.assertFalse(self.controls.is_movement(e3))

    def test_is_movement_up(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_w})
        self.assertTrue(self.controls.is_movement_up(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_UP})
        self.assertTrue(self.controls.is_movement_up(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_i})
        self.assertTrue(self.controls.is_movement_up(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 1, 'value': -1})
        self.assertTrue(self.controls.is_movement_up(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'axis': 1, 'value': -0.5})
        self.assertTrue(self.controls.is_movement_up(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_UP})
        self.assertFalse(self.controls.is_movement_up(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_DOWN})
        self.assertFalse(self.controls.is_movement_up(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 0, 'value': -1})
        self.assertFalse(self.controls.is_movement_up(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 1, 'value': 0})
        self.assertFalse(self.controls.is_movement_up(e))

    def test_is_movement_down(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_s})
        self.assertTrue(self.controls.is_movement_down(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_DOWN})
        self.assertTrue(self.controls.is_movement_down(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_k})
        self.assertTrue(self.controls.is_movement_down(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 1, 'value': 1})
        self.assertTrue(self.controls.is_movement_down(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 1, 'value': 0.5})
        self.assertTrue(self.controls.is_movement_down(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_DOWN})
        self.assertFalse(self.controls.is_movement_down(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_UP})
        self.assertFalse(self.controls.is_movement_down(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 0, 'value': 1})
        self.assertFalse(self.controls.is_movement_down(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 1, 'value': 0})
        self.assertFalse(self.controls.is_movement_down(e))

    def test_is_movement_left(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_a})
        self.assertTrue(self.controls.is_movement_left(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_LEFT})
        self.assertTrue(self.controls.is_movement_left(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_j})
        self.assertTrue(self.controls.is_movement_left(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 0, 'value': -1})
        self.assertTrue(self.controls.is_movement_left(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'axis': 0, 'value': -0.5})
        self.assertTrue(self.controls.is_movement_left(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_a})
        self.assertFalse(self.controls.is_movement_left(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_d})
        self.assertFalse(self.controls.is_movement_left(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 1, 'value': -1})
        self.assertFalse(self.controls.is_movement_left(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 0, 'value': 0})
        self.assertFalse(self.controls.is_movement_left(e))

    def test_is_movement_right(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_d})
        self.assertTrue(self.controls.is_movement_right(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_RIGHT})
        self.assertTrue(self.controls.is_movement_right(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_l})
        self.assertTrue(self.controls.is_movement_right(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 0, 'value': 1})
        self.assertTrue(self.controls.is_movement_right(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 0, 'value': 0.5})
        self.assertTrue(self.controls.is_movement_right(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_d})
        self.assertFalse(self.controls.is_movement_right(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_a})
        self.assertFalse(self.controls.is_movement_right(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 1, 'value': 1})
        self.assertFalse(self.controls.is_movement_right(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {'axis': 0, 'value': 0})
        self.assertFalse(self.controls.is_movement_right(e))

    def test_is_home_button(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_ESCAPE})
        self.assertTrue(self.controls.is_home_button(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_ESCAPE})
        self.assertFalse(self.controls.is_home_button(e))

    def test_is_start_button(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_RETURN})
        self.assertTrue(self.controls.is_start_button(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_b})
        self.assertTrue(self.controls.is_start_button(e))
        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 4})
        self.assertTrue(self.controls.is_start_button(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_RETURN})
        self.assertFalse(self.controls.is_start_button(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_start_button(e))
        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 0})
        self.assertFalse(self.controls.is_start_button(e))
        e = pygame.event.Event(pygame.JOYBUTTONUP, {'button': 0})
        self.assertFalse(self.controls.is_start_button(e))

    def test_is_blue_button(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_q})
        self.assertTrue(self.controls.is_blue_button(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_t})
        self.assertTrue(self.controls.is_blue_button(e))
        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 2})
        self.assertTrue(self.controls.is_blue_button(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_q})
        self.assertFalse(self.controls.is_blue_button(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_blue_button(e))
        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 0})
        self.assertFalse(self.controls.is_blue_button(e))
        e = pygame.event.Event(pygame.JOYBUTTONUP, {'button': 2})
        self.assertFalse(self.controls.is_blue_button(e))

    def test_is_yellow_button(self):
        # Tests that should return true

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_e})
        self.assertTrue(self.controls.is_yellow_button(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_u})
        self.assertTrue(self.controls.is_yellow_button(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_SPACE})
        self.assertTrue(self.controls.is_yellow_button(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 3})
        self.assertTrue(self.controls.is_yellow_button(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_e})
        self.assertFalse(self.controls.is_yellow_button(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_yellow_button(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 1})
        self.assertFalse(self.controls.is_yellow_button(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {'button': 3})
        self.assertFalse(self.controls.is_yellow_button(e))

    def test_is_red_button(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_x})
        self.assertTrue(self.controls.is_red_button(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_m})
        self.assertTrue(self.controls.is_red_button(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 1})
        self.assertTrue(self.controls.is_red_button(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_x})
        self.assertFalse(self.controls.is_red_button(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_red_button(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 6})
        self.assertFalse(self.controls.is_red_button(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {'button': 1})
        self.assertFalse(self.controls.is_red_button(e))

    def test_is_black_button(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_f})
        self.assertTrue(self.controls.is_black_button(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_n})
        self.assertTrue(self.controls.is_black_button(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 0})
        self.assertTrue(self.controls.is_black_button(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_f})
        self.assertFalse(self.controls.is_black_button(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_black_button(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {'button': 6})
        self.assertFalse(self.controls.is_black_button(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {'button': 0})
        self.assertFalse(self.controls.is_black_button(e))

    # Tests for Player 1
    def test_is_movement_up_1(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_w})
        self.assertTrue(self.controls.is_movement_up_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 1, 'value': -1})
        self.assertTrue(self.controls.is_movement_up_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 1, 'value': -0.5})
        self.assertTrue(self.controls.is_movement_up_1(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_w})
        self.assertFalse(self.controls.is_movement_up_1(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_s})
        self.assertFalse(self.controls.is_movement_up_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 0, 'value': -1})
        self.assertFalse(self.controls.is_movement_up_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 1, 'value': 0})
        self.assertFalse(self.controls.is_movement_up_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': -1})
        self.assertFalse(self.controls.is_movement_up_1(e))

    def test_is_movement_down_1(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_s})
        self.assertTrue(self.controls.is_movement_down_1(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 1, 'value': 1})
        self.assertTrue(self.controls.is_movement_down_1(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 1, 'value': 0.5})
        self.assertTrue(self.controls.is_movement_down_1(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_s})
        self.assertFalse(self.controls.is_movement_down_1(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_w})
        self.assertFalse(self.controls.is_movement_down_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 0, 'value': 1})
        self.assertFalse(self.controls.is_movement_down_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 1, 'value': 0})
        self.assertFalse(self.controls.is_movement_down_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': 1})
        self.assertFalse(self.controls.is_movement_down_1(e))

    def test_is_movement_left_1(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_a})
        self.assertTrue(self.controls.is_movement_left_1(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 0, 'value': -1})
        self.assertTrue(self.controls.is_movement_left_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 0, 'value': -0.5})
        self.assertTrue(self.controls.is_movement_left_1(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_a})
        self.assertFalse(self.controls.is_movement_left_1(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_d})
        self.assertFalse(self.controls.is_movement_left_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 1, 'value': -1})
        self.assertFalse(self.controls.is_movement_left_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 0, 'value': 0})
        self.assertFalse(self.controls.is_movement_left_1(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': -1})
        self.assertFalse(self.controls.is_movement_left_1(e))

    def test_is_movement_right_1(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_d})
        self.assertTrue(self.controls.is_movement_right_1(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 0, 'value': 1})
        self.assertTrue(self.controls.is_movement_right_1(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 0, 'value': 0.5})
        self.assertTrue(self.controls.is_movement_right_1(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_d})
        self.assertFalse(self.controls.is_movement_right_1(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_a})
        self.assertFalse(self.controls.is_movement_right_1(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 1, 'value': 1})
        self.assertFalse(self.controls.is_movement_right_1(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1001, 'axis': 0, 'value': 0})
        self.assertFalse(self.controls.is_movement_right_1(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': 1})
        self.assertFalse(self.controls.is_movement_right_1(e))

    def test_is_blue_button_1(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_q})
        self.assertTrue(self.controls.is_blue_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1001, 'button': 2})
        self.assertTrue(self.controls.is_blue_button_1(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_q})
        self.assertFalse(self.controls.is_blue_button_1(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_blue_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1001, 'button': 0})
        self.assertFalse(self.controls.is_blue_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {
                               'instance_id': 1001, 'button': 2})
        self.assertFalse(self.controls.is_blue_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 2})
        self.assertFalse(self.controls.is_blue_button_1(e))

    def test_is_yellow_button_1(self):
        # Tests that should return true

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_e})
        self.assertTrue(self.controls.is_yellow_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1001, 'button': 3})
        self.assertTrue(self.controls.is_yellow_button_1(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_e})
        self.assertFalse(self.controls.is_yellow_button_1(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_yellow_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1001, 'button': 1})
        self.assertFalse(self.controls.is_yellow_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {
                               'instance_id': 1001, 'button': 3})
        self.assertFalse(self.controls.is_yellow_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 3})
        self.assertFalse(self.controls.is_yellow_button_1(e))

    def test_is_red_button_1(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_x})
        self.assertTrue(self.controls.is_red_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1001, 'button': 1})
        self.assertTrue(self.controls.is_red_button_1(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_x})
        self.assertFalse(self.controls.is_red_button_1(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_red_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1001, 'button': 6})
        self.assertFalse(self.controls.is_red_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {
                               'instance_id': 1001, 'button': 1})
        self.assertFalse(self.controls.is_red_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 1})
        self.assertFalse(self.controls.is_red_button_1(e))

    def test_is_black_button_1(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_f})
        self.assertTrue(self.controls.is_black_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1001, 'button': 0})
        self.assertTrue(self.controls.is_black_button_1(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_f})
        self.assertFalse(self.controls.is_black_button_1(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_black_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1001, 'button': 6})
        self.assertFalse(self.controls.is_black_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {
                               'instance_id': 1001, 'button': 0})
        self.assertFalse(self.controls.is_black_button_1(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 0})
        self.assertFalse(self.controls.is_black_button_1(e))

    # Tests for Player 2
    def test_is_movement_up_2(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_i})
        self.assertTrue(self.controls.is_movement_up_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': -1})
        self.assertTrue(self.controls.is_movement_up_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': -0.5})
        self.assertTrue(self.controls.is_movement_up_2(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_i})
        self.assertFalse(self.controls.is_movement_up_2(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_k})
        self.assertFalse(self.controls.is_movement_up_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': -1})
        self.assertFalse(self.controls.is_movement_up_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': 0})
        self.assertFalse(self.controls.is_movement_up_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1003, 'axis': 1, 'value': -1})
        self.assertFalse(self.controls.is_movement_up_2(e))

    def test_is_movement_down_2(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_k})
        self.assertTrue(self.controls.is_movement_down_2(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': 1})
        self.assertTrue(self.controls.is_movement_down_2(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': 0.5})
        self.assertTrue(self.controls.is_movement_down_2(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_k})
        self.assertFalse(self.controls.is_movement_down_2(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_i})
        self.assertFalse(self.controls.is_movement_down_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': 1})
        self.assertFalse(self.controls.is_movement_down_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': 0})
        self.assertFalse(self.controls.is_movement_down_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1003, 'axis': 1, 'value': 1})
        self.assertFalse(self.controls.is_movement_down_2(e))

    def test_is_movement_left_2(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_j})
        self.assertTrue(self.controls.is_movement_left_2(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': -1})
        self.assertTrue(self.controls.is_movement_left_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': -0.5})
        self.assertTrue(self.controls.is_movement_left_2(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_j})
        self.assertFalse(self.controls.is_movement_left_2(e))
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_l})
        self.assertFalse(self.controls.is_movement_left_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': -1})
        self.assertFalse(self.controls.is_movement_left_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': 0})
        self.assertFalse(self.controls.is_movement_left_2(e))
        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1003, 'axis': 0, 'value': -1})
        self.assertFalse(self.controls.is_movement_left_2(e))

    def test_is_movement_right_2(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_l})
        self.assertTrue(self.controls.is_movement_right_2(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': 1})
        self.assertTrue(self.controls.is_movement_right_2(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': 0.5})
        self.assertTrue(self.controls.is_movement_right_2(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_l})
        self.assertFalse(self.controls.is_movement_right_2(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_a})
        self.assertFalse(self.controls.is_movement_right_2(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 1, 'value': 1})
        self.assertFalse(self.controls.is_movement_right_2(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1002, 'axis': 0, 'value': 0})
        self.assertFalse(self.controls.is_movement_right_2(e))

        e = pygame.event.Event(pygame.JOYAXISMOTION, {
                               'instance_id': 1003, 'axis': 0, 'value': 1})
        self.assertFalse(self.controls.is_movement_right_2(e))

    def test_is_blue_button_2(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_t})
        self.assertTrue(self.controls.is_blue_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 2})
        self.assertTrue(self.controls.is_blue_button_2(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_t})
        self.assertFalse(self.controls.is_blue_button_2(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_blue_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 0})
        self.assertFalse(self.controls.is_blue_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {
                               'instance_id': 1002, 'button': 2})
        self.assertFalse(self.controls.is_blue_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1003, 'button': 2})
        self.assertFalse(self.controls.is_blue_button_2(e))

    def test_is_yellow_button_2(self):
        # Tests that should return true

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_u})
        self.assertTrue(self.controls.is_yellow_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 3})
        self.assertTrue(self.controls.is_yellow_button_2(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_u})
        self.assertFalse(self.controls.is_yellow_button_2(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_yellow_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 1})
        self.assertFalse(self.controls.is_yellow_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {
                               'instance_id': 1002, 'button': 3})
        self.assertFalse(self.controls.is_yellow_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1003, 'button': 3})
        self.assertFalse(self.controls.is_yellow_button_2(e))

    def test_is_red_button_2(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_m})
        self.assertTrue(self.controls.is_red_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 1})
        self.assertTrue(self.controls.is_red_button_2(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_m})
        self.assertFalse(self.controls.is_red_button_2(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_red_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 6})
        self.assertFalse(self.controls.is_red_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {
                               'instance_id': 1002, 'button': 1})
        self.assertFalse(self.controls.is_red_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1003, 'button': 1})
        self.assertFalse(self.controls.is_red_button_2(e))

    def test_is_black_button_2(self):
        # Tests that should return true
        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_n})
        self.assertTrue(self.controls.is_black_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 0})
        self.assertTrue(self.controls.is_black_button_2(e))

        # Tests that should return false
        e = pygame.event.Event(pygame.KEYUP, {'key': pygame.K_n})
        self.assertFalse(self.controls.is_black_button_2(e))

        e = pygame.event.Event(pygame.KEYDOWN, {'key': pygame.K_F1})
        self.assertFalse(self.controls.is_black_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1002, 'button': 6})
        self.assertFalse(self.controls.is_black_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONUP, {
                               'instance_id': 1002, 'button': 0})
        self.assertFalse(self.controls.is_black_button_2(e))

        e = pygame.event.Event(pygame.JOYBUTTONDOWN, {
                               'instance_id': 1003, 'button': 0})
        self.assertFalse(self.controls.is_black_button_2(e))


if __name__ == '__main__':
    unittest.main()
